@@ -25,7 +25,7 @@ class InMemCapture(Capture):
 
     def __init__(self, bpf_filter=None, display_filter=None, only_summaries=False,
                  decryption_key=None, encryption_type='wpa-pwk', decode_as=None,
-                 disable_protocol=None, tshark_path=None, override_prefs=None, use_json=False,
+                 disable_protocol=None, tshark_path=None, override_prefs=None, use_json=False, use_ek=False,
                  linktype=LinkTypes.ETHERNET, include_raw=False, eventloop=None, custom_parameters=None,
                  debug=False):
         """Creates a new in-mem capture, a capture capable of receiving binary packets and parsing them using tshark.
@@ -45,12 +45,14 @@ class InMemCapture(Capture):
         :param override_prefs: A dictionary of tshark preferences to override, {PREFERENCE_NAME: PREFERENCE_VALUE, ...}.
         :param disable_protocol: Tells tshark to remove a dissector for a specifc protocol.
         :param custom_parameters: A dict of custom parameters to pass to tshark, i.e. {"--param": "value"}
+        or else a list of parameters in the format ["--foo", "bar", "--baz", "foo"].
         """
         super(InMemCapture, self).__init__(display_filter=display_filter, only_summaries=only_summaries,
                                            decryption_key=decryption_key, encryption_type=encryption_type,
                                            decode_as=decode_as, disable_protocol=disable_protocol,
                                            tshark_path=tshark_path, override_prefs=override_prefs,
-                                           use_json=use_json, include_raw=include_raw, eventloop=eventloop,
+                                           use_json=use_json, use_ek=use_ek,
+                                           include_raw=include_raw, eventloop=eventloop,
                                            custom_parameters=custom_parameters, debug=debug)
         self.bpf_filter = bpf_filter
         self._packets_to_write = None
